import os
import openai
from typing import List
from rich.console import Console
from rich.prompt import Prompt

from langchain.chat_models import ChatOpenAI

from skyagi.context import Context
from skyagi.simulation.agent import GenerativeAgent
from skyagi.simulation.simulation import create_new_memory_retriever, run_conversation, interview_agent


# whether amy wants to talk to bob based on the observations
def talks_to(amy: GenerativeAgent, bob: GenerativeAgent, observations: List[str]) -> str:
    instruct = "Here are the timeline of events happend for these NPC characters:\n"
    instruct += "\n".join(observations)
    instruct += "\n"
    instruct += f"I want to you to behave as {amy.name} and talk to me as I am {bob.name}.\n"
    instruct += f"If you do not want to or can not talk to {bob.name}, just output NOTHING"

    prompts = [
        {"role": "system", "content": f"You are the AI behind a NPC character called {amy.name}"},
        {"role": "user", "content": instruct},
    ]
    resp = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        messages=prompts
    )
    message = resp["choices"][0]["message"]["content"]
    if "NOTHING" in message:
        return ""

    prompts.append(resp["choices"][0]["message"])
    prompts.append({"role": "user", "content": f"Did {amy.name} talk to {bob.name}, please answer yes or no"})
    resp = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        messages=prompts
    )
    if "no" in resp["choices"][0]["message"]["content"]:
        return ""
    return message

def user_robot_conversation(agent_to_interview: GenerativeAgent, ctx: Context):
    ctx.console.print(f"Interview with {agent_to_interview.name} start, input empty line to exit", style="yellow")
    ctx.observations.append(f"{ctx.user_agent.name} now is having a conversation with {agent_to_interview.name}")
    while True:
        user_message = Prompt.ask()
        if user_message == "":
            ctx.console.print(f"Interview with {agent_to_interview.name} finished")
            break
        ctx.observations.append(f"{ctx.user_agent.name} said: {user_message}")
        response = interview_agent(agent_to_interview, user_message, ctx.user_agent.name)
        ctx.console.print(response)
        ctx.observations.append(response)
    ctx.observations.append(f"The conversation between {ctx.user_agent.name} and {agent_to_interview.name} is ended.")


def agi_step(ctx: Context, instruction: dict) -> None:
    ctx.clock += 1
    if instruction["command"] == "interview":
        user_robot_conversation(instruction["agent_to_interview"], ctx)

    if instruction["command"] == "continue":
        someone_asked = False
        for robot_agent in ctx.robot_agents:
            message = talks_to(robot_agent, ctx.user_agent, ctx.observations)
            if message:
                if someone_asked:
                    ctx.console.print(f"{robot_agent.name} also whispered to you({ctx.user_agent.name}): {message}", style="yellow")
                else:
                    ctx.console.print(f"{robot_agent.name} whispered to you ({ctx.user_agent.name}): {message}", style="yellow")
                someone_asked = True
                respond = Prompt.ask(f"Do you want to respond to {robot_agent.name}?", choices=["yes", "no"], default="yes")
                if respond == "yes":
                    user_robot_conversation(robot_agent, ctx)

    # let the activities of non user robots happen
    for idx in range(len(ctx.robot_agents)-1):
        amy = ctx.robot_agents[idx]
        for bob in ctx.robot_agents[idx+1:]:
            message = talks_to(amy, bob, ctx.observations)
            if message:
                ctx.console.print(f"{amy.name} just whispered to {bob.name}...", style="yellow")
                run_conversation([amy, bob], f"{amy.name} said: {message}", ctx)
                ctx.console.print(f"{amy.name} and {bob.name} finished their private conversation...", style="yellow")
                continue
            message = talks_to(bob, amy, ctx.observations)
            if message:
                ctx.console.print(f"{bob.name} just whispered to {amy.name}...", style="yellow")
                run_conversation([bob, amy], f"{bob.name} said: {message}", ctx)
                ctx.console.print(f"{bob.name} and {amy.name} finished their private conversation...", style="yellow")
                continue

    # clean up context's observations based on the time window
    ctx.observations_size_history.append(len(ctx.observations))
    if len(ctx.observations_size_history) > ctx.timewindow_size:
        remove_count = ctx.observations_size_history[0]
        ctx.observations = ctx.observations[remove_count:]
        ctx.observations_size_history = ctx.observations_size_history[1:]
        for idx in range(len(ctx.observations_size_history)):
            ctx.observations_size_history[idx] = ctx.observations_size_history[idx] - remove_count

def agi_init(agent_configs: List[dict], console: Console, openai_key: str, user_idx: int = 0) -> Context:
    ctx = Context(console, openai_key)
    os.environ['OPENAI_API_KEY'] = openai_key
    for idx, agent_config in enumerate(agent_configs):
        agent_name = agent_config["name"]
        console.print(f"Creating agent {agent_name}", style="yellow")
        agent = GenerativeAgent(
            name=agent_config["name"],
            age=agent_config["age"],
            traits=agent_config["personality"],
            status="N/A",  # When connected to a virtual world, we can have the characters update their status
            memory_retriever=create_new_memory_retriever(),
            llm=ChatOpenAI(max_tokens=1500),
            daily_summaries=[
                (
                    agent_config["current_status"]
                )
            ],
            reflection_threshold=8,
        )
        for memory in agent_config["memories"]:
            agent.add_memory(memory)
        if idx == user_idx:
            ctx.user_agent = agent
        else:
            ctx.robot_agents.append(agent)
        ctx.agents.append(agent)
        ctx.observations.append(agent_config["current_status"])
        console.print(f"Agent {agent_name} successfully created", style="green")

    console.print("SkyAGI started...")
    console.print(f"You are going to behave as {ctx.user_agent.name}", style="yellow")
    return ctx



from datetime import datetime, timedelta

from termcolor import colored

from langchain.chat_models import ChatOpenAI

from skyagi.simulation.agent import GenerativeAgent
from skyagi.simulation.simulation import create_new_memory_retriever, run_conversation, interview_agent

LLM = ChatOpenAI(max_tokens=1500)  # Can be any LLM you want.

tommie = GenerativeAgent(
    name="Tommie",
    age=25,
    traits="anxious, likes design",  # You can add more persistent traits here
    status="looking for a job",  # When connected to a virtual world, we can have the characters update their status
    memory_retriever=create_new_memory_retriever(),
    llm=LLM,
    daily_summaries=[
        "Drove across state to move to a new town but doesn't have a job yet."
    ],
    reflection_threshold=8,  # we will give this a relatively low number to show how reflection works
)

print(tommie.get_summary())

tommie_memories = [
    "Tommie remembers his dog, Bruno, from when he was a kid",
    "Tommie feels tired from driving so far",
    "Tommie sees the new home",
    "The new neighbors have a cat",
    "The road is noisy at night",
    "Tommie is hungry",
    "Tommie tries to get some rest.",
]

for memory in tommie_memories:
    tommie.add_memory(memory)

print(tommie.get_summary(force_refresh=True))

# Let's have Tommie start going through a day in the life.
observations = [
    "Tommie wakes up to the sound of a noisy construction site outside his window.",
    "Tommie gets out of bed and heads to the kitchen to make himself some coffee.",
    "Tommie realizes he forgot to buy coffee filters and starts rummaging through his moving boxes to find some.",
    "Tommie finally finds the filters and makes himself a cup of coffee.",
    "The coffee tastes bitter, and Tommie regrets not buying a better brand.",
    "Tommie checks his email and sees that he has no job offers yet.",
    "Tommie spends some time updating his resume and cover letter.",
    "Tommie heads out to explore the city and look for job openings.",
    "Tommie sees a sign for a job fair and decides to attend.",
    "The line to get in is long, and Tommie has to wait for an hour.",
    "Tommie meets several potential employers at the job fair but doesn't receive any offers.",
    "Tommie leaves the job fair feeling disappointed.",
    "Tommie stops by a local diner to grab some lunch.",
    "The service is slow, and Tommie has to wait for 30 minutes to get his food.",
    "Tommie overhears a conversation at the next table about a job opening.",
    "Tommie asks the diners about the job opening and gets some information about the company.",
    "Tommie decides to apply for the job and sends his resume and cover letter.",
    "Tommie continues his search for job openings and drops off his resume at several local businesses.",
    "Tommie takes a break from his job search to go for a walk in a nearby park.",
    "A dog approaches and licks Tommie's feet, and he pets it for a few minutes.",
    "Tommie sees a group of people playing frisbee and decides to join in.",
    "Tommie has fun playing frisbee but gets hit in the face with the frisbee and hurts his nose.",
    "Tommie goes back to his apartment to rest for a bit.",
    "A raccoon tore open the trash bag outside his apartment, and the garbage is all over the floor.",
    "Tommie starts to feel frustrated with his job search.",
    "Tommie calls his best friend to vent about his struggles.",
    "Tommie's friend offers some words of encouragement and tells him to keep trying.",
    "Tommie feels slightly better after talking to his friend.",
]

# Let's send Tommie on their way. We'll check in on their summary every few observations to watch it evolve
for i, observation in enumerate(observations):
    _, reaction = tommie.generate_reaction(observation)
    print(colored(observation, "green"), reaction)
    if ((i + 1) % 20) == 0:
        print("*" * 40)
        print(
            colored(
                f"After {i+1} observations, Tommie's summary is:\n{tommie.get_summary(force_refresh=True)}",
                "blue",
            )
        )
        print("*" * 40)

eve = GenerativeAgent(
    name="Eve",
    age=34,
    traits="curious, helpful",  # You can add more persistent traits here
    status="N/A",  # When connected to a virtual world, we can have the characters update their status
    memory_retriever=create_new_memory_retriever(),
    llm=LLM,
    daily_summaries=[
        (
            "Eve started her new job as a career counselor last week and received her first assignment, a client named Tommie."
        )
    ],
    reflection_threshold=5,
)

yesterday = (datetime.now() - timedelta(days=1)).strftime("%A %B %d")
eve_memories = [
    "Eve overhears her colleague say something about a new client being hard to work with",
    "Eve wakes up and hear's the alarm",
    "Eve eats a boal of porridge",
    "Eve helps a coworker on a task",
    "Eve plays tennis with her friend Xu before going to work",
    "Eve overhears her colleague say something about Tommie being hard to work with",
]
for memory in eve_memories:
    eve.add_memory(memory)


agents = [tommie, eve]
run_conversation(
    agents,
    "Tommie said: Hi, Eve. Thanks for agreeing to share your story with me and give me advice. I have a bunch of questions.",
)

print(tommie.get_summary(force_refresh=True))
print(eve.get_summary(force_refresh=True))



import os
import json
import typer
import time
from pathlib import Path
from rich.console import Console
from rich.prompt import Prompt, IntPrompt

from skyagi import config, util
from skyagi.skyagi import agi_step, agi_init

cli = typer.Typer()
console = Console()

#######################################################################################
# Config CLI

config_cli = typer.Typer()


@config_cli.command("openai")
def config_openai():
    """
    Configure OpenAI API token
    """
    token = Prompt.ask("Enter your OpenAI API token").strip()
    verify_resp = util.verify_openai_token(token)
    if verify_resp != "OK":
        console.print("[Error] OpenAI Token is invalid", style="red")
        console.print(verify_resp)
        return
    config.set_openai_token(token)
    console.print("OpenAI Key is Configured Successfully!", style="green")


@config_cli.command("pinecone")
def config_pinecone():
    """
    Configure Pinecone API token
    """
    token = Prompt.ask("Enter your Pinecone API token").strip()
    verify_resp = util.verify_pinecone_token(token)
    if verify_resp != "OK":
        console.print("[Error] Pinecone Token is invalid", style="red")
        console.print(verify_resp)
        return
    config.set_pinecone_token(token)
    console.print("Pinecone Key is Configured Successfully!", style="green")


@config_cli.callback(invoke_without_command=True)
def config_main(ctx: typer.Context):
    """
    Configure SkyAGI
    """
    # only run without a command specified
    if ctx.invoked_subcommand is not None:
        return

    console.print("SkyAGI's Configuration")

    if (config.load_openai_token()):
        console.print("OpenAI Token is configured, good job!", style="green")
    else:
        console.print("OpenAI Token not configured yet! This is necessary to use SkyAGI", style="red")
        console.print("To config OpenAI token: [yellow]skyagi config openai[/yellow]")

    if (config.load_pinecone_token()):
        console.print("Pinecone Token is configured, good job!", style="green")
    else:
        console.print("Pinecone Token not configured yet! This is necessary to use SkyAGI", style="red")
        console.print("To config Pinecone token: [yellow]skyagi config pinecone[/yellow]")

cli.add_typer(config_cli, name="config")

#######################################################################################
# Main CLI

@cli.command("status")
def status():
    """
    Status of the SkyAGI
    """
    console.print("SkyAGI status")


@cli.command("run")
def run():
    """
    Run SkyAGI
    """
    # Verify the OpenAI token before anything else
    verify_openai = util.verify_openai_token(config.load_openai_token())
    if verify_openai != "OK":
        console.print("Please config your OpenAI token before using this app", style="red")
        console.print("Config by running: skyagi run openai", style="yellow")
        console.print(verify_openai)
        return
    # Get inputs from the user
    agent_count = IntPrompt.ask("Number of agents to create?", default=3)
    agent_configs = []
    agent_names = []
    for idx in range(agent_count):
        console.print(f"Specify agent number {idx+1}")
        agent_config = {}
        while True:
            agent_file = Prompt.ask("Enter the path to the agent configuration file", default="./agent.json")
            if not os.path.isfile(agent_file):
                console.print(f"Invalid file path: {agent_file}", style='red')
                continue
            try:
                agent_config = util.load_json(Path(agent_file))
                if agent_config == {}:
                    console.print(f"Empty configuration, please provide a valid one", style='red')
                    continue
                break
            except json.JSONDecodeError as e:
                console.print(f"Invalid configuration, please provide a valid one", style='red')
                continue
        agent_configs.append(agent_config)
        agent_names.append(agent_config["name"])

    user_role = Prompt.ask("Pick which role you want to perform?", choices=agent_names, default=agent_names[0])
    user_index = agent_names.index(user_role)
    ctx = agi_init(agent_configs, console, config.load_openai_token(), user_index)

    actions = ["continue", "interview", "exit"]
    while True:
        instruction = { "command": "continue" }
        action = Prompt.ask("Pick an action to perform?", choices=actions, default=actions[0])
        if action == "interview":
            robot_agent_names = list(map(lambda agent: agent.name, ctx.robot_agents))
            robot_agent_name = Prompt.ask(f"As {ctx.user_agent.name}, which agent do you want to talk to?", choices=robot_agent_names, default=robot_agent_names[0])
            instruction = {
                "command": "interview",
                "agent_to_interview": ctx.robot_agents[robot_agent_names.index(robot_agent_name)],
            }
        elif action == "exit":
            console.print("SkyAGI exiting...", style="yellow")
            break
        agi_step(ctx, instruction)


@cli.callback(invoke_without_command=True)
def main(
    ctx: typer.Context,
):
    """
    Default behavior
    """
    # only run without a command specified
    if ctx.invoked_subcommand is not None:
        return

    run()
